@@ -4,8 +4,7 @@
 import re
 import shlex
 import time
-import xmpp
-import xmpp.simplexml
+import xmppony as xmpp
 import sqlalchemy.ext.declarative
 import sqlalchemy.orm.exc
 import sqlalchemy.orm
@@ -225,6 +224,34 @@ def qin():
         kind = 'balanced'
     return "%s (%s)" % (ret, kind)
 
+def l5r(pool, adds, target, unskilled=False, emphasized=False):
+    rolls = []
+    for die in range(pool):
+        val = rand(1, 10)
+        if emphasized:
+            while val == 1:
+                val = rand(1, 10)
+        amt = val
+        if not unskilled:
+            while val == 10:
+                val = rand(1, 10)
+                amt += val
+        rolls.append(amt)
+    rolls.sort()
+    res = sum(rolls[-adds:]) - target
+    if res < 0:
+        return "Failure."
+    return "Success (%s)" % res
+
+def fudge(num):
+    res = sum(rand(-1, 1) for i in xrange(num))
+    if res < 0:
+        return "%s" % res
+    if res == 0:
+        return "0"
+    if res > 0:
+        return "+%s" % res
+
 def ork(dice):
     res = defaultdict(int)
     for i in xrange(dice):
@@ -531,6 +558,7 @@ class DiceBot(Bot):
                 "allflesh",
                 "qin",
                 "orkworld",
+                "l5r",
                 "wushu",
                 "alternity",
                 "innomine",
@@ -562,6 +590,8 @@ class DiceBot(Bot):
             * All Flesh Must Be Eaten: roll #
                 skill
             * Qin: roll
+            * Legend of the 5 Rings: #k# # (u) (e)
+                pool, adds, target number, unskilled, emphasized
             * Orkworld: roll #
                 pool size
             * Wushu: roll # over #
@@ -577,13 +607,13 @@ class DiceBot(Bot):
             * Heaven and Earth: roll # (-)# (#)
                 heaven, earth, passing grade
             * Generic Dice: roll #d#
-                number, size"""
+                number, size (size may be F for Fudge dice)"""
         args = msg.getBody()
         try:
             command, args = args.split(None, 1)
         except:
             pass
-        _generic = re.compile(r'(\d*)d(\d+)')
+        _generic = re.compile(r'(\d*)d(\d+|F|f)')
         _rick = re.compile(r'rick')
         if self.mode == "owod":
             _owod = re.compile(r'^(\d+) at (\d+)( s)?( w)?$')
@@ -641,6 +671,18 @@ class DiceBot(Bot):
             _qin = re.compile(r'^$')
             if _qin.search(args):
                 return qin()
+        if self.mode == "l5r":
+            _l5r = re.compile(r'^(\d+)k(\d+) (\d+)( u)?( e)?$')
+            if _l5r.search(args):
+                pool, adds, target, unskilled, emphasized = \
+                                                _l5r.search(args).groups()
+                try:
+                    pool = int(pool)
+                    adds = int(adds)
+                    target = int(target)
+                except ValueError, e:
+                    return "Bad value: %s" % e
+                return l5r(pool, adds, target, unskilled, emphasized)
         if self.mode == "ork":
             _ork = re.compile(r'^(\d+)$')
             if _ork.search(args):
@@ -720,6 +762,12 @@ class DiceBot(Bot):
             num, size = _generic.search(args).groups()
             if num == '':
                 num = 1
+            if size == 'F' or size == 'f':
+                try:
+                    num = int(num)
+                except ValueError, e:
+                    return "Bad number: %s" % e
+                return fudge(num)
             try:
                 num = int(num)
                 size = int(size)
